/*
 ## Cypress USB 3.0 Platform header file (cyfxuvcinmem.h)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

#ifndef _INCLUDED_CYFXUVCINMEM_H_
#define _INCLUDED_CYFXUVCINMEM_H_

#include <cyu3externcstart.h>
#include <cyu3types.h>
#include <cyu3usbconst.h>

/* This header file comprises of the UVC application constants and
 * the video frame configurations */

#define UVC_APP_THREAD_STACK           (0x1000)        /* Thread stack size */
#define UVC_APP_THREAD_PRIORITY        (8)             /* Thread priority */

#define  CY_FX_USBUART_DMA_BUF_COUNT      (2)
#define  CY_FX_USBUART_THREAD_STACK       (1000)
#define  CY_FX_USBUART_THREAD_PRIORITY     (8)

/* Endpoint definition for UVC application */
#define CY_FX_EP_BULK_VIDEO            (0x83)          /* EP 1 IN configured as Bulk EP */
#define CY_FX_EP_VIDEO_CONS_SOCKET     (0x03)	//(CY_U3P_UIB_SOCKET_CONS_1) /* Consumer socket 1 */
#define CY_FX_EP_CONTROL_STATUS        (0x82)          /* EP 2 IN */

/*Endpoint definition for UART application */
#define CY_FX_EP_INTERRUPT              0x84           /* EP 3 INTR */
#define CY_FX_EP_PRODUCER               0x05           /* EP 5 OUT */
#define CY_FX_EP_CONSUMER               0x85           /* EP 5 IN */

#define CY_FX_EP_PRODUCER1_SOCKET        CY_U3P_UIB_SOCKET_PROD_5
#define CY_FX_EP_CONSUMER1_SOCKET        CY_U3P_LPP_SOCKET_UART_CONS
#define CY_FX_EP_PRODUCER2_SOCKET        CY_U3P_LPP_SOCKET_UART_PROD
#define CY_FX_EP_CONSUMER2_SOCKET        CY_U3P_UIB_SOCKET_CONS_5
#define CY_FX_EP_INTR_CONSUMER1_SOCKET   CY_U3P_UIB_SOCKET_CONS_4

#define CY_FX_EP_BULK_VIDEO_PKT_SIZE    (0x400)         /* 1024 Bytes */

/* UVC Video Streaming Endpoint Packet Count */
#define CY_FX_EP_BULK_VIDEO_PKTS_COUNT  (0x10)          /* 16 packets (burst of 16) per DMA buffer. */

/* DMA buffer size used for video streaming. */
#define CY_FX_UVC_STREAM_BUF_SIZE       (CY_FX_EP_BULK_VIDEO_PKTS_COUNT * CY_FX_EP_BULK_VIDEO_PKT_SIZE)  /* 16 KB */

/* Maximum video data that can be accommodated in one DMA buffer. */
#define CY_FX_UVC_BUF_FULL_SIZE         (CY_FX_UVC_STREAM_BUF_SIZE - 16)

/* UVC descriptor types */
#define CY_FX_INTF_ASSN_DSCR_TYPE      (11)            /* Interface association descriptor type. */

/* UVC Buffer count */
#define CY_FX_UVC_STREAM_BUF_COUNT     (8)

#define CY_FX_UVC_MAX_HEADER           (12)         /* Maximum number of header bytes in UVC */
#define CY_FX_UVC_HEADER_DEFAULT_BFH   (0x8C)       /* Default BFH(Bit Field Header) for the UVC Header */

#define CY_FX_UVC_MAX_PROBE_SETTING    (34)         /* Maximum number of bytes in Probe Control */
#define CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED    (64) /* Maximum number of bytes in Probe Control aligned to 32 byte */

#define CY_FX_UVC_HEADER_FRAME          (0)                     /* Normal frame indication */
#define CY_FX_UVC_HEADER_EOF            (uint8_t)(1 << 1)       /* End of frame indication */
#define CY_FX_UVC_HEADER_FRAME_ID       (uint8_t)(1 << 0)       /* Frame ID toggle bit */

#define CY_FX_UVC_INTERFACE_VC          (0)                     /* Video Control interface id. */
#define CY_FX_UVC_INTERFACE_VS          (1)                     /* Video Streaming interface id. */



#define CY_FX_UVC_STREAM_EVENT                  (1 << 0)
#define CY_FX_UVC_STREAM_ABORT_EVENT            (1 << 1)
#define CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT   (1 << 2)
#define CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT    (1 << 3)
#define CY_FX_UVC_DMA_RESET_EVENT               (1 << 4)
#define CY_FX_USB_SUSPEND_EVENT_HANDLER         (1 << 5)

#define CY_FX_USB_UVC_VS_PROBE_CONTROL  (0x0100)                /* Control selector for VS_PROBE_CONTROL. */
#define CY_FX_USB_UVC_VS_COMMIT_CONTROL (0x0200)                /* Control selector for VS_COMMIT_CONTROL. */

#define CY_FX_USB_UVC_VC_RQT_ERROR_CODE_CONTROL (0x0200)
#define CY_FX_USB_UVC_RQT_STAT_INVALID_CTRL     (0x06)
#define CY_FX_UVC_MAX_COMMIT_BUF_FAILURE_CNT    (30)


#define SET_LINE_CODING        0x20
#define GET_LINE_CODING        0x21
#define SET_CONTROL_LINE_STATE 0x22
#define CY_FX_UVC_INVALID_GPIF_STATE    (257)
#define CY_FX_UVC_INTERFACE_CTRL        (uint8_t)(0)            /* wIndex value used to select UVC interface control. */
#define CY_FX_UVC_CAMERA_TERMINAL_ID    (uint8_t)(1)            /* wIndex value used to select Camera terminal. */
#define CY_FX_UVC_PROCESSING_UNIT_ID    (uint8_t)(2)            /* wIndex value used to select Processing Unit. */
#define CY_FX_UVC_EXTENSION_UNIT_ID     (uint8_t)(3)            /* wIndex value used to select Extension Unit. */
#define CY_FX_UVC_STREAM_INTERFACE      (uint8_t)(1)            /* Streaming Interface : Alternate Setting 1 */
#define CY_FX_UVC_CONTROL_INTERFACE     (uint8_t)(0)            /* Control Interface */
#define CY_FX_UVC_PROBE_CTRL            (uint16_t)(0x0100)      /* wValue setting used to access PROBE control. */
#define CY_FX_UVC_COMMIT_CTRL           (uint16_t)(0x0200)      /* wValue setting used to access COMMIT control. */
#define CY_FX_USB_UVC_SET_REQ_TYPE      (uint8_t)(0x21)         /* UVC Interface SET Request Type */
#define CY_FX_USB_UVC_GET_REQ_TYPE      (uint8_t)(0xA1)         /* UVC Interface GET Request Type */
#define CY_FX_USB_UVC_GET_CUR_REQ       (uint8_t)(0x81)         /* UVC GET_CUR Request */
#define CY_FX_USB_UVC_SET_CUR_REQ       (uint8_t)(0x01)         /* UVC SET_CUR Request */
#define CY_FX_USB_UVC_GET_MIN_REQ       (uint8_t)(0x82)         /* UVC GET_MIN Request */
#define CY_FX_USB_UVC_GET_MAX_REQ       (uint8_t)(0x83)         /* UVC GET_MAX Request */
#define CY_FX_USB_UVC_GET_RES_REQ       (uint8_t)(0x84)         /* UVC GET_RES Request */
#define CY_FX_USB_UVC_GET_LEN_REQ       (uint8_t)(0x85)         /* UVC GET_LEN Request */
#define CY_FX_USB_UVC_GET_INFO_REQ      (uint8_t)(0x86)         /* UVC GET_INFO Request */
#define CY_FX_USB_UVC_GET_DEF_REQ       (uint8_t)(0x87)         /* UVC GET_DEF Request */
#define CY_FX_UVC_PU_BACKLIGHT_COMPENSATION_CONTROL         (uint16_t)(0x0100)
#define CY_FX_UVC_PU_BRIGHTNESS_CONTROL                     (uint16_t)(0x0200)
#define CY_FX_UVC_PU_CONTRAST_CONTROL                       (uint16_t)(0x0300)
#define CY_FX_UVC_PU_GAIN_CONTROL                           (uint16_t)(0x0400)
#define CY_FX_UVC_PU_POWER_LINE_FREQUENCY_CONTROL           (uint16_t)(0x0500)
#define CY_FX_UVC_PU_HUE_CONTROL                            (uint16_t)(0x0600)
#define CY_FX_UVC_PU_SATURATION_CONTROL                     (uint16_t)(0x0700)
#define CY_FX_UVC_PU_SHARPNESS_CONTROL                      (uint16_t)(0x0800)
#define CY_FX_UVC_PU_GAMMA_CONTROL                          (uint16_t)(0x0900)
#define CY_FX_UVC_PU_WHITE_BALANCE_TEMPERATURE_CONTROL      (uint16_t)(0x0A00)
#define CY_FX_UVC_PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL (uint16_t)(0x0B00)
#define CY_FX_UVC_PU_WHITE_BALANCE_COMPONENT_CONTROL        (uint16_t)(0x0C00)
#define CY_FX_UVC_PU_WHITE_BALANCE_COMPONENT_AUTO_CONTROL   (uint16_t)(0x0D00)
#define CY_FX_UVC_PU_DIGITAL_MULTIPLIER_CONTROL             (uint16_t)(0x0E00)
#define CY_FX_UVC_PU_DIGITAL_MULTIPLIER_LIMIT_CONTROL       (uint16_t)(0x0F00)
#define CY_FX_UVC_PU_HUE_AUTO_CONTROL                       (uint16_t)(0x1000)
#define CY_FX_UVC_PU_ANALOG_VIDEO_STANDARD_CONTROL          (uint16_t)(0x1100)
#define CY_FX_UVC_PU_ANALOG_LOCK_STATUS_CONTROL             (uint16_t)(0x1200)
#define CY_FX_UVC_VC_ERROR_CODE_NO_ERROR                    (0x00)
#define CY_FX_UVC_VC_ERROR_CODE_NOT_READY                   (0x01)
#define CY_FX_UVC_VC_ERROR_CODE_WRONG_STATE                 (0x02)
#define CY_FX_UVC_VC_ERROR_CODE_POWER                       (0x03)
#define CY_FX_UVC_VC_ERROR_CODE_OUT_OF_RANGE                (0x04)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_UNIT                (0x05)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_CONTROL             (0x06)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_REQUEST             (0x07)
#define CY_FX_UVC_VC_ERROR_CODE_INVALID_VAL_IN_RANGE        (0x08)
#define CY_FX_UVC_VC_ERROR_CODE_UNKNOWN                     (0xFF)
#define CY_FX_UVC_VC_REQUEST_ERROR_CODE_CONTROL             (uint16_t)(0x0200)
#define CY_FX_USB_SET_INTF_REQ_TYPE     (uint8_t)(0x01)         /* USB SET_INTERFACE Request Type. */
#define CY_FX_USB_SET_INTERFACE_REQ     (uint8_t)(0x0B)         /* USB SET_INTERFACE Request code. */

#define CY_FX_USB_SETUP_INDEX_MASK      (uint32_t)(0x0000FFFF)  /* Mask for wIndex field from a control request. */
#define CY_FX_USB_SETUP_LENGTH_MASK     (uint32_t)(0xFFFF0000)  /* Mask for wLength field from a control request. */


#define FPS_120	0x85, 0x45, 0x01, 0x00
#define FPS_90	0x07, 0xb2, 0x01, 0x00
#define FPS_60	0x0a, 0x8b, 0x02, 0x00
#define FPS_30	0x15, 0x16, 0x05, 0x00

#define IMAGE_WIDTH_800		0x20, 0x03
#define IMAGE_WIDTH_720		0xd0, 0x02
#define IMAGE_HEIGHT_480 	0xe0, 0x01
#define IMAGE_HEIGHT_482 	0xe2, 0x01

#define IMAGE_FPS		FPS_120
#define IMAGE_BPP		16
/*
 * 800x480x30
 * 800x482x30
 * 800x480x120
 * 800x482x120


*/

#define DEBUG_UART


/* Extern definitions of the USB Enumeration constant arrays used for the Application */

/* Extern definitions of the Video frame data */

/* UVC Probe Control Setting */
 
/* Video frame lengths */

/* MJPEG Video Frames */
typedef enum CyFxUvcDmaResetVal
{
    CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE = 0,           /* FX3 DMA reset event haven't occurred */
    CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE,          /* FX3 DMA reset event caused due to a commit buffer failure */
    CY_FX_UVC_DMA_RESET_FRAME_TIMER_OVERFLOW            /* FX3 DMA reset event caused due to frame timer overflow */
} CyFxUvcDmaResetVal_t;

#define CY_FX_RQT_ID_CHECK                      (0xB0)
#define CY_FX_RQT_I2C_EEPROM_WRITE              (0xBA)
#define CY_FX_RQT_I2C_EEPROM_READ               (0xBB)
#define CY_FX_RQT_SYS_MEM_READ                  (0xC0)
#define CY_FX_RQT_GET_FW_VERSION                (0xC8)
#define CY_FX_RQT_BOOTLOADER					(0xA0)
#define CY_FX_RQT_CHECK_PROGRAMMER				(0xEE)

#define CY_FX_FLASH_PROG_TIMEOUT                (1000)


#include <cyu3externcend.h>

typedef struct {
	uint32_t data0;
	uint32_t data1;
} request_t;

#endif /* _INCLUDED_CYFXUVCINMEM_H_ */

/*[]*/

